'''
Created on Sep 20, 2019 @author: levente.marton
'''

import os
from time import sleep
from datetime import datetime
from datetime import date
from collections import deque

from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import ui as sui
from selenium.webdriver.common.by import By
from rich.console import Console

from pybank._remotebank import RemoteBank
from pybank._exceptions import no_element_exceptions, all_element_exceptions, safety_timeout_exc
from pybank import mailer
from pybank.bacx_logger import extras_logger_bacx
from pybank.bacx_logger import balance_logger_bacx

def print(*objects, end: str="\n"):
    console = Console()
    console.print(*objects, style='deep_sky_blue1', end=end)


letter = mailer.mailer_details(mail_to=mailer.TO, mail_from=mailer.FR_ADDR, mail_subj=mailer.SUBJ)

xpaths = {'url': 'https://ro.unicreditbanking.net/disp?lang=ro&embedcontentname=BusinessNet',
          'uni_el_us': 'login_input mandatory_field',
          'uni_el_pw': '1259368392',
          'uni_login': '1138725LEVENTE',  # '1138725MZK',
          'uni_refresh': ':w.label',
          'uni_hist': ':n.label',
          'd_name': 'UNI',
          'els': '//*[@id]',
          'sel_per': '//*[@id]/input',
          'search_els': '//*[@id]/div/div/input',
          'exp_csv': '//*[@id]/div/div/div[3]/a/span',
          'sold': '//*[@id]/tr[1]/td[6]/div[1]'
          }

class Bank(RemoteBank):
    '''
    classdocs
    '''
    def __init__(self):
        '''
        Constructor
        '''
        super().__init__()

    def bank_login(self):
        self.switch_to.window(self.window_handles[0])
        # self.delete_all_cookies()
        # print('deleted all cookies')
        self.get(xpaths['url'])
        print('...bank url is %s' % (xpaths['url'])); sleep(1)
        els = self.find_elements(By.XPATH, xpaths['els'])
        for i in els:
            if i.get_attribute('class') == xpaths['uni_el_us']:
                e = i.get_attribute('id')
                user = self.find_element(By.ID, e)
        user.send_keys(xpaths['uni_login'])
        user.send_keys(Keys.RETURN); sleep(5)
        els = self.find_elements(By.XPATH, xpaths['els'])
        for i in els:
            if i.get_attribute('class') == xpaths['uni_el_us']:
                e = i.get_attribute('id')
                passv = self.find_element(By.ID, e)
        passv.send_keys(input('...token pass here - '))
        passv.send_keys(Keys.RETURN)

    def _extras_dl(self, balance):
        print('verifying statements'); sleep(1)
        with open(self.statement_folder) as log_file:
            if len(os.listdir('pybank/extrase')) < 1:
                print('folder is empty')
            else:
                pass
            lines = log_file.readlines()
            if len(lines) < 1:
                print('file is empty')
            else:
                print('files were found', lines[-1][:10])
            if datetime.now().hour > 17 and lines[-1][:10] != str(datetime.now().date()):
                els = self.find_elements(By.XPATH, xpaths['els'])
                for extr in els:
                    if extr.get_attribute('class') == 'pActionIcon headpdf':
                        extr_id = extr.get_attribute('id')
                        extr_dl = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.ID, extr_id))
                        extr_dl.click()
                        sleep(0.5)
                print('statement downloaded')
                extras_logger_bacx.info(balance)
            else:
                print('skipping statement'); sleep(2)

    def _export_csv(self):
        sleep(1.5)
        hist = self.find_element(By.ID, xpaths['uni_hist']); sleep(1.5)
        hist.click(); sleep(3)
        sel_per = self.find_elements(By.XPATH, xpaths['sel_per']); sleep(1.5)
        for per in sel_per:
            if per.get_attribute('class') == 'input_radio' and per.get_attribute('value') == 'FROM_TO':
                # print(per.get_attribute('class'))
                per.click(); sleep(1.5)
        search_els = self.find_elements(By.XPATH, xpaths['search_els'])
        for search in search_els:
            if search.get_attribute('class') == 'button_filter' and search.get_attribute('value') == 'Search':
                # print(search.get_attribute('class'))
                search.click(); sleep(1.5)
                break
        search_els = self.find_elements(By.XPATH, xpaths['search_els'])
        for exp in search_els:
            if exp.get_attribute('class') == 'button_action' and exp.get_attribute('value') == 'Export':
                # print(exp.get_attribute('class'))
                exp.click(); sleep(1.5)
                break
        # self.get('https://ro.unicreditbanking.net/disp/export.csv?INF_DOWNLOAD&dee8$d@=export.csv')
        sleep(1.5)
        exp_csv = self.find_elements(By.XPATH, xpaths['exp_csv'])
        # print(len(exp_csv))
        for file in exp_csv:
            file.click(); sleep(1.5)
            break
        print('csv exported', end='\n')
        # exp_csv.click(); sleep(1.5)

    def _transaction_screenshot(self):
        empl_pattern = r'[A-Za-z]* [A-Za-z@:-]*(?=,)'
        ops = []
        sleep(3)
        # transaction = self.find_elements(By.CLASS_NAME, 'first')[1]
        first = sui.WebDriverWait(self, 4).until(lambda browser: browser.find_elements(By.CLASS_NAME, 'first'))[1]
        ops.append(first.text)
        even = sui.WebDriverWait(self, 4).until(lambda browser: browser.find_elements(By.CLASS_NAME, 'even'))
        even = even[1:]
        odd = sui.WebDriverWait(self, 4).until(lambda browser: browser.find_elements(By.CLASS_NAME, 'odd'))
        odd = odd[1:]
        # ops.append(first.text)
        today_ = date.today().strftime('%d.%m.%Y')
        if len(odd) >= len(even):
            for i in range(len(odd)):
                    try:
                        empl = mailer.re_check(even[i].text, pattern=empl_pattern)
                        if today_ in even[i].text and empl not in self.employees:
                            ops.append(even[i].text)
                    except IndexError:
                        pass
                    empl = mailer.re_check(odd[i].text, pattern=empl_pattern)
                    if today_ in odd[i].text and empl not in self.employees:
                        ops.append(odd[i].text)
        else:
            for i in range(len(even)):
                empl = mailer.re_check(even[i].text, pattern=empl_pattern)
                if today_ in even[i].text and empl not in self.employees:
                    ops.append(even[i].text)
                try:
                    empl = mailer.re_check(odd[i].text, pattern=empl_pattern)
                    if today_ in odd[i].text and empl not in self.employees:
                        ops.append(odd[i].text)
                except IndexError:
                    pass
        # print(transaction.text)
        self.transaction = mailer.Transactions(ops)
        # self.transaction = first.text
        sleep(1)
        return self.transaction

    def refresh(self):
        c = 1
        err = 1; sleep(15)
        bal = self.find_element(By.ID, xpaths['uni_refresh'])
        print(bal.get_attribute('id'))
        sold_cont = deque(maxlen=2)
        logback = False
        first_start = True
        # all_els = self.find_elements(By.XPATH, '//*[@id]/tr[2]/td[6]/div[1]')
        # print(all_els[0].text)
        while True:
            # self.transaction = 'Confidential content'
            # s_cont = []
            try:
                t = 210; sleep(3)
                print(xpaths['d_name'])
                # with open(self.statement_filename()) as statement:
                with open('pybank/extrase/BACX.log') as statement:
                    if first_start:
                        sold = mailer.dig_it(str(statement.readlines()[-1][11:]))
                        if logback is False or len(sold_cont) < 2:
                            sold_cont.appendleft(sold)
                            print(type(sold_cont[0]), sold_cont[0])
                    else:
                        sold = self.find_elements(By.XPATH, xpaths['sold'])
                        sold = sold[0].text
                        if logback is False:
                            sold_cont.appendleft(sold)
                logback = False
                # s_cont.append(sold)
                try:
                    self._extras_dl(sold_cont[0])
                    bal.click(); sleep(2)
                except all_element_exceptions:
                    print('...skipping')
                if isinstance(mailer.dig_it(sold_cont[0], sold_cont[0]), float):
                    print('...prev balance: {}'.format(mailer.dig_it(sold_cont[0], sold_cont[0])))
                while t >= 0:
                    mins, secs = divmod(t, 60)
                    timeformat = '{:02d}:{:02d}'.format(mins, secs)
                    print('...clicking in', timeformat, end='\r'); sleep(1)
                    t -= 1
                print('')
                print('...now clicking')
                bal.click(); sleep(3)
                sold1 = self.find_elements(By.XPATH, xpaths['sold'])  # '//*[@id]/tr[2]/td[6]/div[1]'
                sold1 = sold1[0].text
                print('')
                try:
                    sleep(0.5)
                    if isinstance(mailer.dig_it(sold1.text, sold_cont[0]), float):
                        print('...new balance: {0}'.format(mailer.dig_it(sold1.text, sold_cont[0])))
                except (ValueError, AttributeError):
                    sleep(1)
                    if isinstance(mailer.dig_it(sold1, sold_cont[0]), float):
                        print('...new balance2: {0}'.format(mailer.dig_it(sold1, sold_cont[0])))
                print('2`nd try')
                # NEW
                to_send = mailer.dig_it(sold1, sold_cont[0]) - mailer.dig_it(sold_cont[0], sold_cont[0])
                if to_send != 0:
                    if isinstance(mailer.dig_it(sold1, sold_cont[0]), float):
                        balance_logger_bacx.info('{:.2f}'.format(mailer.dig_it(sold1, sold_cont[0])))
                        print('balance logged')
                        # if to_send > 0:
                        self._transaction_screenshot()
                        subject = xpaths['d_name'] + ' ' + mailer.SUBJ + \
                            ' {0:2d}:{1:2d}:{2:2d}'.format(datetime.now().time().hour, datetime.now().time().minute, datetime.now().time().second)
                        body = (
                            'Bank name: {0}\nTransaction details: {1}\n\n{2}\n\n{3}'.format(
                                xpaths['d_name'], round(to_send, 2), datetime.now().time(), repr(self.transaction)))
                        mailer.yag.send(to=mailer.TO, subject=subject, contents=body)
                        print('...notification sent, diff: {0}'.format(to_send), end='\n')
                        self._export_csv()
                        # print('csv exported', end='\n')
                        # balance_logger_bacx.info('{:.2f}'.format(mailer.dig_it(sold1, sold_cont[0])))
                        # print('balance logged')
                else:
                    if isinstance(mailer.dig_it(sold1, sold_cont[0]), float):
                        print('...balance not changed, diff: {0}'.format(to_send))
                        balance_logger_bacx.info('{:.2f}'.format(mailer.dig_it(sold1, sold_cont[0])))
                        print('balance logged')
                print('...clicked {0} times'.format(c))
                c += 1
            except all_element_exceptions:
                print('...element lost %s times, waiting to return' % (err))
                if err > 1:
                    logback = True
                try:
                    bal = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.ID, xpaths['uni_refresh']))  # browser.find_element(By.ID, data_dict[to_refresh])
                    # print(bal.get_attribute('id'))
                except safety_timeout_exc:
                        self.bank_login()
                        logback = True
                err += 1
            # first_start = False
