'''
Created on Jul 9, 2019 @author: levente.marton
'''

import os
from time import sleep
from datetime import datetime
from datetime import date
from collections import deque

from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import ui as sui
from selenium.webdriver.common.by import By
from rich.console import Console

from pybank._remotebank import RemoteBank
from pybank._exceptions import no_element_exceptions, all_element_exceptions, safety_timeout_exc
from pybank import mailer
from pybank.brd_logger import extras_logger_brd
from pybank.brd_logger import balance_logger_brd

def print(*objects, end: str="\n"):
    console = Console()
    console.print(*objects, style='red', end=end)


letter = mailer.mailer_details(mail_to=mailer.TO, mail_from=mailer.FR_ADDR, mail_subj=mailer.SUBJ)

xpaths = {'url': 'https://www.brdoffice.ro/smartoffice/_mcologon?..OASLogon%2ELanguage=RO',
          'el_us': 'OASLogon.User',
          'el_sign_table': '//*[@id="SignTable"]/div[2]/div/div/input',
          'el_pw': 'OASLogon.PIN',
          'login': '31302059',   # or '31300126'
          'spass': os.getenv('BRDIB'),
          'refresh': 'pmi_1_125',   # or 'pmi_1_123'
          'd_name': 'BRD',
          'close_ad': 'splash-45555-close-button',
          'sign_table': '//*[@id="SignTable"]/div[2]/div/div/input',
          'rap': 'pmi_1',
          # 'extr': '//*[@id="idListUngrouped"]/tbody/tr[3]/td[1]/a',
          'extr_dl': '//*[@id="mainview_cnt"]/div[2]/div[2]/a',
          'backward': '//*[@id="mainview_cnt"]/div[2]/div[1]/a[3]',
          'account': '//*[@id="idListUngrouped"]/tbody/tr[3]/td[1]/a',
          'transactions': 'splitter',
          'sold': '//*[@id="idListUngrouped"]/tbody/tr[3]/td[6]'
          }

class Bank(RemoteBank):
    '''
    classdocs
    '''
    def __init__(self):
        '''
        Constructor
        '''
        super().__init__()

    def bank_login(self, token_passw=None):
        self.switch_to.window(self.window_handles[0])
        # print('...switching to tab %s' % (tab + 1))
        # self.delete_all_cookies()
        # print('deleted all cookies')
        self.get(xpaths['url'])
        print('...bank url is %s' % (xpaths['url'])); sleep(1)
        user = self.find_element(By.ID, xpaths['el_us'])
        passv = self.find_element(By.ID, xpaths['el_pw'])
        try:
            close_ad = self.find_element(By.ID, xpaths['close_ad'])
            close_ad.click()
        except no_element_exceptions as _exc:
            print('no ad found')
        finally:
            user.send_keys(xpaths['login']); sleep(1.5)
            # token_passw = input('...token pass here')
            secret_passw = xpaths['spass']
            # passv.send_keys(token_passw)
            passv.send_keys(secret_passw); sleep(1.5)
            passv.send_keys(Keys.RETURN); sleep(1.5)
            sign_table = self.find_element(By.XPATH, xpaths['sign_table'])
            m_token_passw = input('...token pass here - ')
            sign_table.send_keys(m_token_passw)
            sign_table.send_keys(Keys.RETURN); sleep(1.5)
        rap = sui.WebDriverWait(self, 15).until(lambda browser: browser.find_element(By.ID, xpaths['rap']))
        rap.click(); sleep(3)

    def _extras_dl(self, balance):
        print('verifying statements')
        with open(self.statement_folder) as log_file:
            if len(os.listdir('pybank/extrase')) < 1:
                print('folder is empty')
            else:
                # for att in filter(lambda files: files.endswith('.pdf'), os.listdir('extrase')):
                pass
            lines = log_file.readlines()
            if len(lines) < 1:
                print('file is empty')
            else:
                print('files were found', lines[-1][:10])
            if datetime.now().hour > 17 and lines[-1][:10] != str(datetime.now().date()):
                try:
                    extr = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['account']))
                    extr.click(); sleep(1.5)
                    # extr_dl = self.find_element(By.ID, 'linkpdf')
                    extr_dl = self.find_element(By.XPATH, xpaths['extr_dl'])
                    extr_dl.click(); sleep(1.5)
                    backward = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['backward']))
                    backward.click(); sleep(1.5)
                finally:
                    print('extras_dl performed'); sleep(0.5)
                print('statement downloaded')
                extras_logger_brd.info(balance)
            else:
                print('skipping statement'); sleep(2)

    def _transaction_screenshot(self):
        sleep(3)
        empl_pattern = r'(?<=Dna )[A-Za-z]* [A-Za-z@:-]*_*[A-Za-z@:-]*(?= )|(?<=Dl )[A-Za-z]* [A-Za-z@:-]*_*[A-Za-z@:-]*(?= )'
        ops = []
        try:
            account = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['account']))
            account.click(); sleep(3)
        finally:
            print('prepearing screenshot'); sleep(0.5)
        # transaction = sui.WebDriverWait(self, 4).until(lambda browser: browser.find_elements(By.CLASS_NAME, 'altRow')[0])
        # transaction = sui.WebDriverWait(self, 4).until(lambda browser: browser.find_elements(By.ID, 'splitter')[-1])
        transactions = sui.WebDriverWait(self, 4).until(lambda browser: browser.find_elements(By.ID, xpaths['transactions']))
        # if len(transactions) > 1:
        today_ = date.today().strftime('%d.%m.%Y')
        for transaction in transactions:
            empl = mailer.re_check(transaction.text, pattern=empl_pattern)
            if today_ in transaction.text and empl not in self.employees:
                ops.append(transaction.text)
        ops = list(reversed(ops))
        # self.transaction = transaction.text
        self.transaction = mailer.Transactions(ops)
        sleep(1)
        backward = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['backward']))
        backward.click(); sleep(1)
        return self.transaction

    def refresh(self):
        c = 1
        err = 1
        bal = self.find_element(By.ID, xpaths['refresh'])
        print(bal.get_attribute('id'))
        sold_cont = deque(maxlen=2)
        logback = False
        first_start = True
        while True:
            # self.transaction = 'Confidential content'
            # s_cont = []
            try:
                t = 210; sleep(3)
                print(xpaths['d_name'])
                # with open(self.statement_folder) as statement:
                with open('pybank/extrase/BRD.log') as statement:
                    if first_start:
                        sold = mailer.dig_it(str(statement.readlines()[-1][11:]))
                        if logback is False or len(sold_cont) < 2:
                            sold_cont.appendleft(sold)
                            print(type(sold_cont[0]), sold_cont[0])
                    else:
                        sold = self.find_element(By.XPATH, xpaths['sold'])
                        while sold.text == 'Neactualizat' or sold.text == 'Not updated':
                            bal.click()
                            sold = self.find_element(By.XPATH, xpaths['sold'])
                            sleep(2)
                        if logback is False or len(sold_cont) < 2:
                            sold_cont.appendleft(sold.text)
                # s_cont.append(sold.text)
                logback = False
                try:
                    self._extras_dl(sold_cont[0])
                    bal.click(); sleep(2)
                except all_element_exceptions:
                    print('...skipping')
                # if isinstance(mailer.dig_it(s_cont[0]), float):
                # print(type(mailer.dig_it(str(sold_cont[0]), str(sold_cont[0]))), mailer.dig_it(str(sold_cont[0]), str(sold_cont[0])))
                if isinstance(mailer.dig_it(sold_cont[0], sold_cont[0]), float):
                    print('...prev balance: {}'.format(mailer.dig_it(sold_cont[0], sold_cont[0])))
                while t >= 0:
                    mins, secs = divmod(t, 60)
                    timeformat = '{:02d}:{:02d}'.format(mins, secs)
                    print('...clicking in', timeformat, end='\r'); sleep(1)
                    t -= 1
                print('')
                print('...now clicking')
                bal.click(); sleep(3)
                sold1 = self.find_element(By.XPATH, xpaths['sold'])
                while sold1.text == 'Neactualizat' or sold1.text == 'Not updated':
                    bal.click(); sleep(2)
                    sold1 = self.find_element(By.XPATH, xpaths['sold'])
                    # sleep(0.5)
                print('')
                try:
                    sleep(0.5)
                    if isinstance(mailer.dig_it(sold1.text, sold_cont[0]), float):
                        print('...new balance: {0}'.format(mailer.dig_it(sold1.text, sold_cont[0])))
                except ValueError:
                    sleep(1)
                    try:
                        if isinstance(mailer.dig_it(sold1, sold_cont[0]), float):
                            print('...new balance2: {0}'.format(mailer.dig_it(sold1, sold_cont[0])))
                    except TypeError:
                        print('...{0}'.format(mailer.dig_it(sold1.text, sold_cont[0])))
                # try:
                print('1`st try')
                to_send = mailer.dig_it(sold1.text, sold_cont[0]) - mailer.dig_it(sold_cont[0], sold_cont[0])
                if to_send != 0:
                    if sold1 == '':
                        print('reinitiate sold')
                    else:
                        if isinstance(mailer.dig_it(sold1.text, sold_cont[0]), float):
                            balance_logger_brd.info('{:.2f}'.format(mailer.dig_it(sold1.text, sold_cont[0])))
                            print('balance logged')
                            # if to_send > 0:
                            self._transaction_screenshot()
                            subject = xpaths['d_name'] + ' ' + mailer.SUBJ + \
                                ' {0:2d}:{1:2d}:{2:2d}'.format(datetime.now().time().hour, datetime.now().time().minute, datetime.now().time().second)
                            body = (
                                'Bank name: {0}\nTransaction details: {1}\n\n{2}\n\n{3}'.format(
                                    xpaths['d_name'], round(to_send, 2), datetime.now().time(), repr(self.transaction)))
                            mailer.yag.send(to=mailer.TO, subject=subject, contents=body)
                            print('notification sent, diff: {0}'.format(to_send))
                            # balance_logger_brd.info('{:.2f}'.format(mailer.dig_it(sold1.text, sold_cont[0])))
                            # print('balance logged')
                else:
                    if isinstance(mailer.dig_it(sold1.text, sold_cont[0]), float):
                        print('...balance not changed, diff: {0}'.format(to_send))
                        balance_logger_brd.info('{:.2f}'.format(mailer.dig_it(sold1.text, sold_cont[0])))
                print('...clicked {0} times'.format(c))
                c += 1
            except all_element_exceptions:
                print('...element lost %s times, waiting to return' % (err))
                if err > 1:
                    logback = True
                try:
                    bal = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.ID, xpaths['refresh']))
                except safety_timeout_exc:
                    self.bank_login()
                    logback = True
                # bal.click()  # browser.find_element(By.ID, data_dict[to_refresh])
                err += 1
            # first_start = False
