'''Created Oct 29, 2021 Levi'''

import os
import traceback
import sys
from time import sleep
from datetime import datetime
from datetime import date
from collections import deque

from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import ui as sui
from selenium.webdriver.support import expected_conditions as EC
# from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from rich.console import Console

from pybank._remotebank import RemoteBank
from pybank import mailer
from pybank.btrl_logger import extras_logger_neobt
from pybank.btrl_logger import balance_logger_neobt
from pybank._exceptions import all_element_exceptions, safety_timeout_exc
from pybank._exceptions import StaleElementReferenceException, TimeoutException

ignored_exceptions = (StaleElementReferenceException,)

def print(*objects, end: str="\n"):
    console = Console()
    console.print(*objects, style='yellow', end=end)

def pretty_exc(exc):
    exc_type, _exc_obj, exc_tb = sys.exc_info()
    # fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
    for tb in list(traceback.format_exception(exc_type, _exc_obj, exc_tb)):
        print(tb)


letter = mailer.mailer_details(mail_to=mailer.TO, mail_from=mailer.FR_ADDR, mail_subj=mailer.SUBJ)

xpaths = {'url': 'https://neo.btrl.ro/',
          'el_us': 'MainContentFull_ebLoginControl_txtUserName_txField',
          'el_pw': 'MainContentFull_ebLoginControl_txtCredential_txField',
          'atp': os.getenv('NEOBT'),
          'el_otp': 'MainContentFull_ebLoginControl_txtOTP_txField',
          'login': 'N51904ML',
          'refresh': '//*[@id="MainContent_TransactionMainContent_accControl_rptCurrentAccounts_divAccountType_0"]/div[3]/h5',
          'd_name': 'NEOBT',
          'sold': '//*[@id="MainContent_TransactionMainContent_txpTransactions_ctl01_gvAccountTransactions_lblBalance_0"]',  # '//*[@id="MainContent_TransactionMainContent_accControl_rptCurrentAccounts_divBalance_0"]/h3'
          'client': '//*[@id="MainContentFull_ebSelectControl_SelectList_btnProfile_2"]/div[2]/div[1]',
          'main_account': 'MainContent_TransactionMainContent_BTLandingProductsControl_rptProductCurrentAccount_link_Action2_0',
          'eur_account': '//*[@id="MainContent_TransactionMainContent_accControl_rptCurrentAccounts_divBalance_1"]/h3',
          'usd_account': '//*[@id="MainContent_TransactionMainContent_accControl_rptCurrentAccounts_divBalance_2"]/h3',
          'exp_pdf': '//*[@id="MainContent_TransactionMainContent_txpTransactions_ctl01_proofControl_li_ex_label"]',
          'exp_xml': 'MainContent_TransactionMainContent_txpTransactions_ctl01_proofControl_li_txt_label',
          'filter_btn': 'MainContent_TransactionMainContent_txpTransactions_ctl01_idSearcHeaderLocation',
          'start_date': 'MainContent_TransactionMainContent_txpTransactions_ctl01_dpFromTo_dateFromPicker_txField',
          'end_date': 'MainContent_TransactionMainContent_txpTransactions_ctl01_dpFromTo_dateToPicker_txField',
          'appl_filter_btn': 'MainContent_TransactionMainContent_txpTransactions_ctl01_btnSearch',
          'transactions': '//*[@id="MainContent_TransactionMainContent_txpTransactions_ctl01_gvAccountTransactions"]/tbody/tr',
          'home': '//*[@id="header-container"]/div/div/div/div[1]/div[1]/div[2]/img'
          }

class Bank(RemoteBank):
    '''
    classdocs
    '''
    def __init__(self):
        '''
        Constructor
        '''
        super().__init__()

    def bank_login(self, passw=None):
        # self.delete_all_cookies()
        # print('deleted all cookies')
        self.get(xpaths['url'])
        print('...bank url is %s' % (xpaths['url'])); sleep(1)
        user = self.find_element(By.ID, xpaths['el_us'])
        user.send_keys(xpaths['login']); sleep(1.5)
        passv = self.find_element(By.ID, xpaths['el_pw'])
        passv.send_keys(xpaths['atp']); sleep(1.5)
        passv.send_keys(Keys.RETURN); sleep(1.5)
        otp = self.find_element(By.ID, xpaths['el_otp'])
        otp.send_keys(input('...token pass/OTP here - '))
        otp.send_keys(Keys.RETURN); sleep(3)
        client = self.find_element(By.XPATH, xpaths['client'])
        client.click(); sleep(3)
        main_account = self.find_element(By.ID, xpaths['main_account'])
        main_account.click(); sleep(1.5)

    def _reload_accounts(self):
        # self.delete_all_cookies()
        # print('deleted all cookies')
        print('reloading accounts'); sleep(1)
        neobt = self.find_element(By.XPATH, xpaths['home'])
        # user.send_keys(xpaths['login']); sleep(1.5)
        neobt.click(); sleep(3)
        main_account = self.find_element(By.ID, xpaths['main_account'])
        main_account.click(); sleep(1.5)

    def _extras_dl(self, balance):
        print('verifying statements'); sleep(1)
        with open(self.statement_folder) as log_file:
            if len(os.listdir('pybank/extrase')) < 1:
                print('folder is empty')
            else:
                pass
            lines = log_file.readlines()
            if len(lines) < 1:
                print('file is empty')
            else:
                print('files were found', lines[-1][:10])
            # print(datetime.now().hour)
            if datetime.now().hour > 17 and lines[-1][:10] != str(datetime.now().date()):  # and lines[-1][:10] != str(datetime.now().date())
                eur_bal = self.find_element(By.XPATH, xpaths['eur_account'])
                usd_bal = self.find_element(By.XPATH, xpaths['usd_account'])
                with open('c:/Users/levi/Downloads/neobt_forreign.txt', 'a') as neobt_forreign:
                    neobt_forreign.write(f'BT_EUR: {eur_bal.text}\n')
                    neobt_forreign.write(f'BT_USD: {usd_bal.text}')
                sleep(2)
                extr = sui.WebDriverWait(self, 5).until(lambda browser: browser.find_element(By.XPATH, xpaths['exp_pdf']))  # browser.find_element(By.ID, '26603_link')
                extr.click(); sleep(1)
                print('statement downloaded')
                extras_logger_neobt.info(balance)
            else:
                print('skipping statement'); sleep(2)

    def _export_csv(self):
        exp_but = sui.WebDriverWait(self, 15).until(lambda browser: browser.find_element(By.ID, xpaths['exp_xml']))
        exp_but.click(); sleep(1)
        # alert = Alert(self)
        # alert.accept()

    def _transaction_screenshot(self):
        ops = []
        sleep(3)
        filter_btn = sui.WebDriverWait(self, 15).until(lambda browser: browser.find_element(By.ID, xpaths['filter_btn']))  # './/*[@id="download_1"]/span'
        filter_btn.click(); sleep(0.5)
        start_period = date.today().strftime('%d-%m-%Y')
        start_date = sui.WebDriverWait(self, 15).until(lambda browser: browser.find_element(By.ID, xpaths['start_date']))
        start_date.send_keys(Keys.CONTROL + 'a'); sleep(0.5)
        start_date.send_keys(start_period); sleep(0.5)
        start_date.send_keys(Keys.TAB); sleep(0.5)
        search_but = sui.WebDriverWait(self, 15).until(lambda browser: browser.find_element(By.ID, xpaths['appl_filter_btn']))
        search_but.click(); sleep(5)
        try:
            transaction0 = sui.WebDriverWait(self, 4).until(lambda browser: browser.find_elements(By.XPATH, xpaths['transactions']))
        except safety_timeout_exc:
            pass
            # transaction0 = []
        for i in range(len(transaction0)):
            empl = mailer.re_check(transaction0[i].text)
            if empl not in self.employees:
                ops.append(transaction0[i].text[12:])
        self.transaction = mailer.Transactions(ops)
        self._export_csv()
        print('csv exported', end='\n')
        sleep(1)
        return self.transaction

    def _update_session(self):
        try:
            alert = self.find_element(By.XPATH, '//*[@id="btnyes"]')
            alert.click(); sleep(1)
            self.refresh()
        except Exception:
            pass

    def refresh1(self):
        c = 1
        err = 1; sleep(3)
        bal = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['refresh']))
        print(bal.get_attribute('id'))
        self.sold_cont = deque(maxlen=2)
        logback = False
        first_start = True
        while True:
            try:
                t = 210; sleep(3)
                self.refresh()
                print(xpaths['d_name'])
                with open('pybank/extrase/NEOBT.log') as statement:
                    if first_start:
                        sold = mailer.dig_it(statement.readlines()[-1][11:])
                        self.sold = sold
                        if logback is False or len(self.sold_cont) < 2:
                            self.sold_cont.appendleft(sold)
                            print(type(self.sold_cont[0]), self.sold_cont[0])
                    else:
                        sold = self.find_element(By.XPATH, xpaths['sold'])
                        self.sold = sold.text
                        if logback is False or len(self.sold_cont) < 2:
                            self.sold_cont.appendleft(sold.text)
                logback = False
                try:
                    self._extras_dl(self.sold_cont[0])
                    # bal = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['refresh'])).click()
                    try:
                        self.execute_script('arguments[0].click();', sui.WebDriverWait(self, 20).until(EC.element_to_be_clickable((By.XPATH, xpaths['refresh']))))
                    except all_element_exceptions:  # TimeoutException
                        # self.refresh()
                        self._reload_accounts()
                        self.execute_script('arguments[0].click();', sui.WebDriverWait(self, 20).until(EC.element_to_be_clickable((By.XPATH, xpaths['refresh']))))
                    sleep(5)  # bal.click();
                except all_element_exceptions:
                    print('...skipping')
                if isinstance(mailer.dig_it(self.sold_cont[0], self.sold_cont[0]), float):
                    print('...prev balance: {}'.format(mailer.dig_it(self.sold_cont[0], self.sold_cont[0])))
                else:
                    # bal = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['refresh'])).click()
                    try:
                        self.execute_script('arguments[0].click();', sui.WebDriverWait(self, 20).until(EC.element_to_be_clickable((By.XPATH, xpaths['refresh']))))
                    except all_element_exceptions:  # TimeoutException
                        # self.refresh()
                        self._reload_accounts()
                        self.execute_script('arguments[0].click();', sui.WebDriverWait(self, 20).until(EC.element_to_be_clickable((By.XPATH, xpaths['refresh']))))
                    sleep(5)  # bal.click();
                    sold = self.find_element(By.XPATH, xpaths['sold'])
                    self.sold_cont.appendleft(sold.text)
                while t >= 0:
                    mins, secs = divmod(t, 60)
                    timeformat = '{:02d}:{:02d}'.format(mins, secs)
                    print('...clicking in', timeformat, end='\r'); sleep(1)
                    self._update_session()
                    t -= 1
                # self.refresh()
                print('')
                print('...now clicking')
                # bal = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['refresh'])).click()
                try:
                    self.execute_script('arguments[0].click();', sui.WebDriverWait(self, 20).until(EC.element_to_be_clickable((By.XPATH, xpaths['refresh']))))
                except all_element_exceptions:  # TimeoutException
                    # self.refresh()
                    self._reload_accounts()
                    self.execute_script('arguments[0].click();', sui.WebDriverWait(self, 20).until(EC.element_to_be_clickable((By.XPATH, xpaths['refresh']))))
                sleep(5)  # bal.click();
                while True:
                    try:
                        sold1_text = sui.WebDriverWait(self, 20, ignored_exceptions=ignored_exceptions) \
                            .until(expected_conditions.presence_of_element_located((By.XPATH, xpaths['sold']))).text
                        print('^^balance click success^^')
                        break
                    except StaleElementReferenceException as exc_:
                        pretty_exc(exc_)
                        self._reload_accounts()
                        print('^^retrying in 1 seconds^^')
                        sleep(1)
                        # continue
                try:
                    sleep(0.5)
                    if isinstance(mailer.dig_it(sold1_text, self.sold_cont[0]), float):
                        print('...new balance: {0}'.format(mailer.dig_it(sold1_text, self.sold_cont[0])))
                except ValueError:
                    sleep(1)
                    pass
                    # try:
                    #     if isinstance(mailer.dig_it(sold1, self.sold_cont[0]), float):
                    #         print('...new balance2: {0}'.format(mailer.dig_it(sold1, self.sold_cont[0])))
                    # except TypeError:
                    #     print('...{0}'.format(mailer.dig_it(sold1_text, self.sold_cont[0])))
                print('1`st try')
                to_send = mailer.dig_it(sold1_text, self.sold_cont[0]) - mailer.dig_it(self.sold_cont[0], self.sold_cont[0])
                if to_send != 0:
                    if sold1_text == '':
                        print('reinitiate sold')
                    else:
                        if isinstance(mailer.dig_it(sold1_text, self.sold_cont[0]), float):
                            balance_logger_neobt.info('{:.2f}'.format(mailer.dig_it(sold1_text, self.sold_cont[0])))
                            print('balance logged')
                            # if to_send > 0:
                            self._transaction_screenshot()
                            subject = xpaths['d_name'] + ' ' + mailer.SUBJ + \
                                ' {0:2d}:{1:2d}:{2:2d}'.format(datetime.now().time().hour, datetime.now().time().minute, datetime.now().time().second)
                            body = (
                                'Bank name: {0}\nTransaction details: {1}\n\n{2}\n\n{3}'.format(
                                    xpaths['d_name'], round(to_send, 2), datetime.now().time(), repr(self.transaction)))
                            mailer.yag.send(to=mailer.TO, subject=subject, contents=body)
                            # letter.send(mailer.S_NAME, mailer.P_NUM, mailer.LOG_U, mailer.PAS, mailer.SEC)
                            print('notification sent, diff: {0}'.format(to_send))
                else:
                    if isinstance(mailer.dig_it(sold1_text, self.sold_cont[0]), float):
                        print('...balance not changed, diff: {0}'.format(to_send))
                        balance_logger_neobt.info('{:.2f}'.format(mailer.dig_it(sold1_text, self.sold_cont[0])))
                        print('balance logged')
                print('...clicked {0} times'.format(c))
                c += 1
            except all_element_exceptions as exc_:
                pretty_exc(exc_)
                print('...element lost %s times, waiting to return' % (err))
                if err > 1:
                    logback = True
                try:
                    bal = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['refresh']))
                    sold = self.find_element(By.XPATH, xpaths['sold'])
                    self.sold = sold.text
                except Exception as _:
                    print(str(_))
                    try:
                        bal = sui.WebDriverWait(self, 10).until(lambda browser: browser.find_element(By.XPATH, xpaths['refresh']))
                        sold = self.find_element(By.XPATH, xpaths['sold'])
                        self.sold = sold.text
                    except safety_timeout_exc:
                        sleep(10)
                        self.bank_login()
                        logback = True
                err += 1
